@@ -85,7 +85,7 @@ meta.TextRepository <- function(x, tag, type = NULL) {
     x
 }
 `meta<-.TextDocument` <- function(x, tag, type = NULL, value) {
-    if (tag %in% setdiff(names(attributes(x)), ".Data"))
+    if (tag %in% setdiff(names(attributes(x)), "Content"))
         attr(x, tag) <- value
     else
         attr(x, "LocalMetaData")[[tag]] <- value
@@ -96,6 +96,14 @@ meta.TextRepository <- function(x, tag, type = NULL) {
     x
 }
 
+`content_meta<-` <- function(x, tag, value) {
+    if (identical(tag, "Content"))
+        Content(x) <- value
+    else
+        meta(x, tag) <- value
+    x
+}
+
 # Simple Dublin Core to tm meta data mappings
 # http://en.wikipedia.org/wiki/Dublin_core#Simple_Dublin_Core
 Dublin_Core_tm <-
